@@ -3,11 +3,12 @@ Test the force offscreen auto gui mechanism.
 """
 
 import os
+import gc
 import weakref
 
-import wgpu.backends.rs  # noqa
+import wgpu
 from pytest import fixture, skip
-from testutils import can_use_wgpu_lib
+from testutils import can_use_wgpu_lib, is_pypy
 
 
 if not can_use_wgpu_lib:
@@ -35,9 +36,9 @@ def test_canvas_class():
 
 
 def test_event_loop():
-    """Check that the event loop handles queued tasks and then returns.
-    TODO: This test may hang indefinitely in the failure case, can we
-    prevent that?"""
+    """Check that the event loop handles queued tasks and then returns."""
+    # Note: if this test fails, it may run forever, so it's a good idea to have a timeout on the CI job or something
+
     from wgpu.gui.auto import run, call_later
 
     ran = False
@@ -60,4 +61,6 @@ def test_offscreen_canvas_del():
 
     assert ref() is not None
     del canvas
+    if is_pypy:
+        gc.collect()
     assert ref() is None
