@@ -2,13 +2,29 @@
 Core utilities that are loaded into the root namespace or used internally.
 """
 
+import re
+import sys
+import atexit
 import logging
-from pkg_resources import resource_filename
+import importlib.resources
+from contextlib import ExitStack
+
+
+# Our resources are most probably always on the file system. But in
+# case they don't we have a nice exit handler to remove temporary files.
+_resource_files = ExitStack()
+atexit.register(_resource_files.close)
 
 
 def get_resource_filename(name):
     """Get the filename to a wgpu resource."""
-    return resource_filename("wgpu.resources", name)
+    if sys.version_info < (3, 9):
+        context = importlib.resources.path("wgpu.resources", name)
+    else:
+        ref = importlib.resources.files("wgpu.resources") / name
+        context = importlib.resources.as_file(ref)
+    path = _resource_files.enter_context(context)
+    return str(path)
 
 
 class WGPULogger(logging.getLoggerClass()):
@@ -29,6 +45,47 @@ assert isinstance(logger, WGPULogger)
 logger.setLevel(logging.WARNING)
 
 
+_re_wgpu_ob = re.compile(r"`<[a-z|A-Z]+-\([0-9]+, [0-9]+, [a-z|A-Z]+\)>`")
+
+
+def error_message_hash(message):
+    # Remove wgpu object representations, because they contain id's that may change at each draw.
+    # E.g. `<CommandBuffer- (12, 4, Metal)>`
+    message = _re_wgpu_ob.sub("WGPU_OBJECT", message)
+    return hash(message)
+
+
+_flag_cache = {}  # str -> int
+
+
+def str_flag_to_int(flag, s):
+    """Allow using strings for flags, i.e. 'READ' instead of wgpu.MapMode.READ.
+    No worries about repeated overhead, because the resuls are cached.
+    """
+    cache_key = (
+        f"{flag._name}.{s}"  # using private attribute, lets call this a friend func
+    )
+    value = _flag_cache.get(cache_key, None)
+
+    if value is None:
+        parts = [p.strip() for p in s.split("|")]
+        parts = [p for p in parts if p]
+        invalid_parts = [p for p in parts if p.startswith("_")]
+        if not parts or invalid_parts:
+            raise ValueError(f"Invalid flag value: {s}")
+
+        value = 0
+        for p in parts:
+            try:
+                v = flag.__dict__[p.upper()]
+                value += v
+            except KeyError:
+                raise ValueError(f"Invalid flag value for {flag}: '{p}'")
+        _flag_cache[cache_key] = value
+
+    return value
+
+
 class ApiDiff:
     """Helper class to define differences in the API by annotating
     methods. This way, these difference are made explicit, plus they're
