@@ -7,8 +7,6 @@ On Linux, install the glfw lib using ``sudo apt install libglfw3``,
 or ``sudo apt install libglfw3-wayland`` when using Wayland.
 """
 
-
-import os
 import sys
 import time
 import weakref
@@ -16,7 +14,8 @@ import asyncio
 
 import glfw
 
-from .base import WgpuCanvasBase, WgpuAutoGui, weakbind
+from .base import WgpuCanvasBase, WgpuAutoGui
+from ._gui_utils import SYSTEM_IS_WAYLAND, weakbind, logger
 
 
 # Make sure that glfw is new enough
@@ -26,13 +25,13 @@ if glfw_version_info < (1, 9):
 
 # Do checks to prevent pitfalls on hybrid Xorg/Wayland systems
 is_wayland = False
-if sys.platform.startswith("linux"):
-    is_wayland = "wayland" in os.getenv("XDG_SESSION_TYPE", "").lower()
-    if is_wayland and not hasattr(glfw, "get_wayland_window"):
-        raise RuntimeError(
-            "We're on Wayland but Wayland functions not available. "
-            + "Did you apt install libglfw3-wayland?"
-        )
+if sys.platform.startswith("linux") and SYSTEM_IS_WAYLAND:
+    if not hasattr(glfw, "get_x11_window"):
+        # Probably glfw was imported before we wgpu was, so we missed our chance
+        # to set the env var to make glfw use x11.
+        is_wayland = True
+        logger.warning("Using GLFW with Wayland, which is experimental.")
+
 
 # Some glfw functions are not always available
 set_window_content_scale_callback = lambda *args: None  # noqa: E731
@@ -121,11 +120,6 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
         # Set window hints
         glfw.window_hint(glfw.CLIENT_API, glfw.NO_API)
         glfw.window_hint(glfw.RESIZABLE, True)
-        # see https://github.com/FlorianRhiem/pyGLFW/issues/42
-        # Alternatively, from pyGLFW 1.10 one can set glfw.ERROR_REPORTING='warn'
-        if sys.platform.startswith("linux"):
-            if is_wayland:
-                glfw.window_hint(glfw.FOCUSED, False)  # prevent Wayland focus error
 
         # Create the window (the initial size may not be in logical pixels)
         self._window = glfw.create_window(int(size[0]), int(size[1]), title, None, None)
@@ -142,7 +136,7 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
         # Register callbacks. We may get notified too often, but that's
         # ok, they'll result in a single draw.
         glfw.set_framebuffer_size_callback(self._window, weakbind(self._on_size_change))
-        glfw.set_window_close_callback(self._window, weakbind(self._on_close))
+        glfw.set_window_close_callback(self._window, weakbind(self._check_close))
         glfw.set_window_refresh_callback(self._window, weakbind(self._on_window_dirty))
         glfw.set_window_focus_callback(self._window, weakbind(self._on_window_dirty))
         set_window_content_scale_callback(
@@ -152,8 +146,8 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
         glfw.set_window_iconify_callback(self._window, weakbind(self._on_iconify))
 
         # User input
-        self._key_modifiers = set()
-        self._pointer_buttons = set()
+        self._key_modifiers = []
+        self._pointer_buttons = []
         self._pointer_pos = 0, 0
         self._double_click_state = {"clicks": 0}
         glfw.set_mouse_button_callback(self._window, weakbind(self._on_mouse_button))
@@ -183,10 +177,19 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
         self._determine_size()
         self._request_draw()
 
+    def _check_close(self, *args):
+        # Follow the close flow that glfw intended.
+        # This method can be overloaded and the close-flag can be set to False
+        # using set_window_should_close() if now is not a good time to close.
+        if self._window is not None and glfw.window_should_close(self._window):
+            self._on_close()
+
     def _on_close(self, *args):
         all_glfw_canvases.discard(self)
-        glfw.hide_window(self._window)
-        self._handle_event_and_flush({"event_type": "close"})
+        if self._window is not None:
+            glfw.destroy_window(self._window)  # not just glfw.hide_window
+            self._window = None
+            self._handle_event_and_flush({"event_type": "close"})
 
     def _on_window_dirty(self, *args):
         self._request_draw()
@@ -202,6 +205,8 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
         glfw.post_empty_event()  # Awake the event loop, if it's in wait-mode
 
     def _determine_size(self):
+        if self._window is None:
+            return
         # Because the value of get_window_size is in physical-pixels
         # on some systems and in logical-pixels on other, we use the
         # framebuffer size and pixel ratio to derive the logical size.
@@ -222,6 +227,8 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
         self._handle_event_and_flush(ev)
 
     def _set_logical_size(self, new_logical_size):
+        if self._window is None:
+            return
         # There is unclarity about the window size in "screen pixels".
         # It appears that on Windows and X11 its the same as the
         # framebuffer size, and on macOS it's logical pixels.
@@ -257,27 +264,32 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
 
     # API
 
-    def get_window_id(self):
+    def get_surface_info(self):
         if sys.platform.startswith("win"):
-            return int(glfw.get_win32_window(self._window))
+            return {
+                "platform": "windows",
+                "window": int(glfw.get_win32_window(self._window)),
+            }
         elif sys.platform.startswith("darwin"):
-            return int(glfw.get_cocoa_window(self._window))
+            return {
+                "platform": "cocoa",
+                "window": int(glfw.get_cocoa_window(self._window)),
+            }
         elif sys.platform.startswith("linux"):
             if is_wayland:
-                return int(glfw.get_wayland_window(self._window))
+                return {
+                    "platform": "wayland",
+                    "window": int(glfw.get_wayland_window(self._window)),
+                    "display": int(glfw.get_wayland_display()),
+                }
             else:
-                return int(glfw.get_x11_window(self._window))
+                return {
+                    "platform": "x11",
+                    "window": int(glfw.get_x11_window(self._window)),
+                    "display": int(glfw.get_x11_display()),
+                }
         else:
-            raise RuntimeError(f"Cannot get GLFW window id on {sys.platform}.")
-
-    def get_display_id(self):
-        if sys.platform.startswith("linux"):
-            if is_wayland:
-                return glfw.get_wayland_display()
-            else:
-                return glfw.get_x11_display()
-        else:
-            raise RuntimeError(f"Cannot get GLFW display id on {sys.platform}.")
+            raise RuntimeError(f"Cannot get GLFW surafce info on {sys.platform}.")
 
     def get_pixel_ratio(self):
         return self._pixel_ratio
@@ -299,11 +311,12 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
             call_later(self._get_draw_wait_time(), self._mark_ready_for_draw)
 
     def close(self):
-        glfw.set_window_should_close(self._window, True)
-        self._on_close()
+        if self._window is not None:
+            glfw.set_window_should_close(self._window, True)
+            self._check_close()
 
     def is_closed(self):
-        return glfw.window_should_close(self._window)
+        return self._window is None
 
     # User events
 
@@ -323,10 +336,14 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
 
         if action == glfw.PRESS:
             event_type = "pointer_down"
-            self._pointer_buttons.add(button)
+            buttons = set(self._pointer_buttons)
+            buttons.add(button)
+            self._pointer_buttons = list(sorted(buttons))
         elif action == glfw.RELEASE:
             event_type = "pointer_up"
-            self._pointer_buttons.discard(button)
+            buttons = set(self._pointer_buttons)
+            buttons.discard(button)
+            self._pointer_buttons = list(sorted(buttons))
         else:
             return
 
@@ -426,6 +443,7 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
             "dy": -100.0 * dy,
             "x": self._pointer_pos[0],
             "y": self._pointer_pos[1],
+            "buttons": list(self._pointer_buttons),
             "modifiers": list(self._key_modifiers),
         }
         match_keys = {"modifiers"}
@@ -438,11 +456,15 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
         if action == glfw.PRESS:
             event_type = "key_down"
             if modifier:
-                self._key_modifiers.add(modifier)
+                modifiers = set(self._key_modifiers)
+                modifiers.add(modifier)
+                self._key_modifiers = list(sorted(modifiers))
         elif action == glfw.RELEASE:
             event_type = "key_up"
             if modifier:
-                self._key_modifiers.discard(modifier)
+                modifiers = set(self._key_modifiers)
+                modifiers.discard(modifier)
+                self._key_modifiers = list(sorted(modifiers))
         else:  # glfw.REPEAT
             return
 
