@@ -7,7 +7,11 @@ import sys
 import ctypes
 import importlib
 
-from .base import WgpuCanvasBase, WgpuAutoGui, weakbind
+from .base import WgpuCanvasBase, WgpuAutoGui
+from ._gui_utils import get_alt_x11_display, get_alt_wayland_display, weakbind
+
+
+is_wayland = False  # We force Qt to use X11 in _gui_utils.py
 
 
 # Select GUI toolkit
@@ -17,12 +21,14 @@ for libname in ("PySide6", "PyQt6", "PySide2", "PyQt5"):
         QtWidgets = importlib.import_module(".QtWidgets", libname)
         try:
             WA_PaintOnScreen = QtCore.Qt.WidgetAttribute.WA_PaintOnScreen
+            WA_DeleteOnClose = QtCore.Qt.WidgetAttribute.WA_DeleteOnClose
             PreciseTimer = QtCore.Qt.TimerType.PreciseTimer
             KeyboardModifiers = QtCore.Qt.KeyboardModifier
             FocusPolicy = QtCore.Qt.FocusPolicy
             Keys = QtCore.Qt.Key
         except AttributeError:
             WA_PaintOnScreen = QtCore.Qt.WA_PaintOnScreen
+            WA_DeleteOnClose = QtCore.Qt.WA_DeleteOnClose
             PreciseTimer = QtCore.Qt.PreciseTimer
             KeyboardModifiers = QtCore.Qt
             FocusPolicy = QtCore.Qt
@@ -34,6 +40,16 @@ else:
     )
 
 
+# Get version
+if libname.startswith("PySide"):
+    qt_version_info = QtCore.__version_info__
+else:
+    try:
+        qt_version_info = tuple(int(i) for i in QtCore.QT_VERSION_STR.split(".")[:3])
+    except Exception:  # Failsafe
+        qt_version_info = (0, 0, 0)
+
+
 BUTTON_MAP = {
     QtCore.Qt.MouseButton.LeftButton: 1,  # == MOUSE_BUTTON_LEFT
     QtCore.Qt.MouseButton.RightButton: 2,  # == MOUSE_BUTTON_RIGHT
@@ -90,26 +106,17 @@ KEY_MAP = {
 }
 
 
-# Make Qt not ignore XDG_SESSION_TYPE
-# is_wayland = "wayland" in os.getenv("XDG_SESSION_TYPE", "").lower()
-# if is_wayland:
-#     os.environ["QT_QPA_PLATFORM"] = "wayland"
-
-
 def enable_hidpi():
     """Enable high-res displays."""
-    try:
-        set_dpi_aware = QtCore.__version_info__ < (6, 4)
-    except Exception:
-        set_dpi_aware = True
-    try:
-        # See https://github.com/pyzo/pyzo/pull/700 why we seem to need both
-        # See https://github.com/pygfx/pygfx/issues/368 for high Qt versions
-        if set_dpi_aware:
+    set_dpi_aware = qt_version_info < (6, 4)  # Pyside
+    if set_dpi_aware:
+        try:
+            # See https://github.com/pyzo/pyzo/pull/700 why we seem to need both
+            # See https://github.com/pygfx/pygfx/issues/368 for high Qt versions
             ctypes.windll.shcore.SetProcessDpiAwareness(1)  # global dpi aware
             ctypes.windll.shcore.SetProcessDpiAwareness(2)  # per-monitor dpi aware
-    except Exception:
-        pass  # fail on non-windows
+        except Exception:
+            pass  # fail on non-windows
     try:
         QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
     except Exception:
@@ -131,6 +138,7 @@ class QWgpuWidget(WgpuAutoGui, WgpuCanvasBase, QtWidgets.QWidget):
 
         # Configure how Qt renders this widget
         self.setAttribute(WA_PaintOnScreen, True)
+        self.setAttribute(WA_DeleteOnClose, True)
         self.setAutoFillBackground(False)
         self.setMouseTracking(True)
         self.setFocusPolicy(FocusPolicy.StrongFocus)
@@ -141,11 +149,6 @@ class QWgpuWidget(WgpuAutoGui, WgpuCanvasBase, QtWidgets.QWidget):
         self._request_draw_timer.setSingleShot(True)
         self._request_draw_timer.timeout.connect(self.update)
 
-        # Get the window id one time. For some reason this is needed
-        # to "activate" the canvas. Otherwise the viz is not shown if
-        # one does not provide canvas to request_adapter().
-        self.get_window_id()
-
     def paintEngine(self):  # noqa: N802 - this is a Qt method
         # https://doc.qt.io/qt-5/qt.html#WidgetAttribute-enum  WA_PaintOnScreen
         return None
@@ -155,13 +158,27 @@ class QWgpuWidget(WgpuAutoGui, WgpuCanvasBase, QtWidgets.QWidget):
 
     # Methods that we add from wgpu (snake_case)
 
-    def get_display_id(self):
-        # There is qx11info, but it is rarely available.
-        # https://doc.qt.io/qt-5/qx11info.html#display
-        return super().get_display_id()  # uses X11 lib
-
-    def get_window_id(self):
-        return int(self.winId())
+    def get_surface_info(self):
+        if sys.platform.startswith("win") or sys.platform.startswith("darwin"):
+            return {
+                "window": int(self.winId()),
+            }
+        elif sys.platform.startswith("linux"):
+            # The trick to use an al display pointer works for X11, but results in a segfault on Wayland ...
+            if is_wayland:
+                return {
+                    "platform": "wayland",
+                    "window": int(self.winId()),
+                    "display": int(get_alt_wayland_display()),
+                }
+            else:
+                return {
+                    "platform": "x11",
+                    "window": int(self.winId()),
+                    "display": int(get_alt_x11_display()),
+                }
+        else:
+            raise RuntimeError(f"Cannot get Qt surafce info on {sys.platform}.")
 
     def get_pixel_ratio(self):
         # Observations:
@@ -180,6 +197,7 @@ class QWgpuWidget(WgpuAutoGui, WgpuCanvasBase, QtWidgets.QWidget):
         lsize = self.width(), self.height()
         lsize = float(lsize[0]), float(lsize[1])
         ratio = self.devicePixelRatioF()
+
         # When the ratio is not integer (qt6), we need to somehow round
         # it. It turns out that we need to round it, but also add a
         # small offset. Tested on Win10 with several different OS
@@ -251,8 +269,8 @@ class QWgpuWidget(WgpuAutoGui, WgpuCanvasBase, QtWidgets.QWidget):
         if touches:
             ev.update(
                 {
-                    "ntouches": 0,  # TODO
-                    "touches": {},  # TODO
+                    "ntouches": 0,
+                    "touches": {},  # TODO: Qt touch events
                 }
             )
 
@@ -283,6 +301,11 @@ class QWgpuWidget(WgpuAutoGui, WgpuCanvasBase, QtWidgets.QWidget):
             for mod in MODIFIERS_MAP.keys()
             if mod & event.modifiers()
         ]
+        buttons = [
+            BUTTON_MAP[button]
+            for button in BUTTON_MAP.keys()
+            if button & event.buttons()
+        ]
 
         ev = {
             "event_type": "wheel",
@@ -290,6 +313,7 @@ class QWgpuWidget(WgpuAutoGui, WgpuCanvasBase, QtWidgets.QWidget):
             "dy": -event.angleDelta().y(),
             "x": event.position().x(),
             "y": event.position().y(),
+            "buttons": buttons,
             "modifiers": modifiers,
         }
         match_keys = {"modifiers"}
@@ -324,6 +348,7 @@ class QWgpuCanvas(WgpuAutoGui, WgpuCanvasBase, QtWidgets.QWidget):
 
         super().__init__(**kwargs)
 
+        self.setAttribute(WA_DeleteOnClose, True)
         self.set_logical_size(*(size or (640, 480)))
         self.setWindowTitle(title or "qt wgpu canvas")
         self.setMouseTracking(True)
@@ -331,6 +356,11 @@ class QWgpuCanvas(WgpuAutoGui, WgpuCanvasBase, QtWidgets.QWidget):
         self._subwidget = QWgpuWidget(self, max_fps=max_fps)
         self._subwidget.add_event_handler(weakbind(self.handle_event), "*")
 
+        # Note: At some point we called `self._subwidget.winId()` here. For some
+        # reason this was needed to "activate" the canvas. Otherwise the viz was
+        # not shown if no canvas was provided to request_adapter(). Removed
+        # later because could not reproduce.
+
         layout = QtWidgets.QHBoxLayout(self)
         layout.setContentsMargins(0, 0, 0, 0)
         self.setLayout(layout)
@@ -354,11 +384,8 @@ class QWgpuCanvas(WgpuAutoGui, WgpuCanvasBase, QtWidgets.QWidget):
     def draw_frame(self, f):
         self._subwidget.draw_frame = f
 
-    def get_display_id(self):
-        return self._subwidget.get_display_id()
-
-    def get_window_id(self):
-        return self._subwidget.get_window_id()
+    def get_surface_info(self):
+        return self._subwidget.get_surface_info()
 
     def get_pixel_ratio(self):
         return self._subwidget.get_pixel_ratio()
